/*
  This file uses the GLX protocol to link X11 & OpenGL.
  This file was rewritten by Ellis (laurent_mallet@hotmail.com)
  using some source from Brian Paul.
*/

#include <X11/keysym.h>

#include "../input/stk_keyboard.h"
#include "stk_message.h"
#include "stk_x11.h"



// global variable
Display *stk_x11_display;
Window   stk_x11_window;






static Window create_window(Display *display,
                          int width,
                          int height)
{
   int attrib[] = { GLX_RGBA,
                    GLX_RED_SIZE, 1,
                    GLX_GREEN_SIZE, 1,
                    GLX_BLUE_SIZE, 1,
                    GLX_DOUBLEBUFFER,
                    GLX_DEPTH_SIZE, 1,
                    GLX_STENCIL_SIZE,1,
                    None };
                    
   
   int scrnum;
   XSetWindowAttributes attr;
   unsigned long mask;
   Window root;
   Window win;
   GLXContext ctx;
   XVisualInfo *visinfo;

   scrnum = DefaultScreen( display );
   root = RootWindow( display, scrnum );

   visinfo = glXChooseVisual( display, scrnum, attrib );
   if (!visinfo) {
       printf("Error: couldn't get an RGB, Double-buffered visual\n");
       exit(1);
   }

   /* window attributes */
   attr.background_pixel = 0;
   attr.border_pixel = 0;
   attr.colormap = XCreateColormap( display, root, visinfo->visual, AllocNone);
   attr.event_mask = ExposureMask | StructureNotifyMask |
       KeyPressMask | KeyReleaseMask |
       ButtonPressMask | ButtonReleaseMask|
       ButtonMotionMask | PointerMotionMask;

   mask = CWBackPixel | CWBorderPixel | CWColormap | CWEventMask;

   win = XCreateWindow( display, root, 0, 0, width, height,
		        0, visinfo->depth, InputOutput,
		        visinfo->visual, mask, &attr );

   ctx = glXCreateContext( display, visinfo, NULL, True );

   glXMakeCurrent( display, win, ctx );

   return win;
}



int Stk_SwapBuffers()
{
     glXSwapBuffers( stk_x11_display, stk_x11_window );
     return 0;
}


#define KEYB_MACRO(key,type,pressed)                  \
{                                                     \
     Stk_Message *msg=new Stk_Message(STK_KEY_EVENT); \
     msg->AddInt("type",type);                        \
     msg->AddInt("key",key);                          \
     msg->AddInt("pressed",pressed);                  \
     looper->StkPostMessage(msg);                     \
}






int Stk_EventLoop(Stk_Looper *looper)
{
   XEvent event;

   while(XCheckWindowEvent(stk_x11_display,stk_x11_window, 0xFFFFFFFF,&event)){
   switch (event.type) {
       
       case Expose:
           break;
       case KeyPress:
       {
           KeySym key; XComposeStatus compose;
           char buffer[25]; int buffersize=20;
           XLookupString((XKeyEvent*) &event,buffer,buffersize,&key,&compose);
               // Now we analyse key pressed
           switch(key){
               case XK_Return:
               case XK_KP_Enter:
               case XK_Linefeed:
                   KEYB_MACRO(STK_K_ENTER,2,1);
                   break;
               case XK_BackSpace:
                   KEYB_MACRO(STK_K_BACKSPACE,2,1);
                   break;
               case XK_Delete:
                   KEYB_MACRO(STK_K_DELETE,2,1);
                   break;
               case XK_Escape:
                   KEYB_MACRO(STK_K_ESCAPE,2,1);
                   break;
               case XK_Up:
                   KEYB_MACRO(STK_K_UP,2,1);
                   break;
               case XK_Down:
                   KEYB_MACRO(STK_K_DOWN,2,1);
                   break;
               case XK_Right:
                   KEYB_MACRO(STK_K_RIGHT,2,1);
                   break;
               case XK_Left:
                   KEYB_MACRO(STK_K_LEFT,2,1);
                   break;
               default:
                   KEYB_MACRO(*buffer,1,1);
                       //printf("keysym %s is not handled\n",
                       //   XKeysymToString(key));
                       //buffer[10]=0;
                   break;
           }           
       }
       break;
           
       case KeyRelease:
           break;

       case ButtonPress:
       {          
           Stk_Message *msg=new Stk_Message(STK_MOU_EVENT);
           msg->AddInt("type",1);
           msg->AddInt("button",event.xbutton.button);
           msg->AddInt("pressed",1);
           msg->AddInt("x",event.xbutton.x);
           msg->AddInt("y",event.xbutton.y);
           looper->StkPostMessage(msg);
       }       
       break;
       
       case ButtonRelease:
       {          
           Stk_Message *msg=new Stk_Message(STK_MOU_EVENT);
           msg->AddInt("type",1);
           msg->AddInt("button",event.xbutton.button);
           msg->AddInt("pressed",0);
           msg->AddInt("x",event.xbutton.x);
           msg->AddInt("y",event.xbutton.y);
           looper->StkPostMessage(msg);
       }
       break;
       
       case MotionNotify:
       {          
           Stk_Message *msg=new Stk_Message(STK_MOU_EVENT);
           msg->AddInt("type",2);
           msg->AddInt("x",event.xbutton.x);
           msg->AddInt("y",event.xbutton.y);
           looper->StkPostMessage(msg);
       }   
       break;
   }
   }
   return 0;
}




int Stk_InitGL(int size_x,int size_y)
{
    stk_x11_display = XOpenDisplay(NULL);

    XAutoRepeatOff(stk_x11_display);
         
    stk_x11_window = create_window( stk_x11_display, size_x, size_y );
    
        // Once we have created the window, we draw it
    XMapWindow( stk_x11_display, stk_x11_window );
    return 0;
}


int Stk_QuitGL()
{
    XAutoRepeatOn(stk_x11_display);
    XFlush(stk_x11_display);
    return 0;
}
